package geniusweb.references;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.references.PartyRef;
import geniusweb.references.PartyWithProfile;
import geniusweb.references.ProfileRef;
import tudelft.utilities.junit.GeneralTests;

public class PartyWithProfileTest extends GeneralTests<PartyWithProfile> {

	private PartyRef party1, party1a, party2, party3;
	private ProfileRef profile1, profile2;
	private PartyWithProfile partyprof1, partyprof1a, partyprof2, partyprof3;
	private final String serialized = "{\"party\":\"ws:party1\",\"profile\":\"ws:profile1\"}";

	@Before
	public void before() throws URISyntaxException {
		party1 = new PartyRef(new URI("ws:party1"));
		party1a = new PartyRef(new URI("ws:party1"));
		party2 = new PartyRef(new URI("ws:party2"));
		party3 = new PartyRef(new URI("http:party3"));

		profile1 = new ProfileRef(new URI("ws:profile1"));
		profile2 = new ProfileRef(new URI("ws:profile2"));

		partyprof1 = new PartyWithProfile(party1, profile1);
		partyprof1a = new PartyWithProfile(party1, profile1);
		partyprof2 = new PartyWithProfile(party2, profile1);
		partyprof3 = new PartyWithProfile(party1, profile2);

	}

	@Override
	public List<List<PartyWithProfile>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(partyprof1, partyprof1a),
				Arrays.asList(partyprof2), Arrays.asList(partyprof3));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"PartyWithProfile.PartyRef.ws:party1.,ProfileRef.ws:profile1..",
				"PartyWithProfile.PartyRef.ws:party2.,ProfileRef.ws:profile1..",
				"PartyWithProfile.PartyRef.ws:party1.,ProfileRef.ws:profile2..");
	}

	@Test
	public void smokeTest() throws URISyntaxException {
	}

	@Test(expected = IllegalArgumentException.class)
	public void nullTest() throws URISyntaxException {
		new PartyWithProfile((PartyRef) null, (ProfileRef) null);
	}

	@Test
	public void testSerialize() throws JsonProcessingException {
		ObjectMapper jackson = new ObjectMapper();

		String json = jackson.writeValueAsString(partyprof1);
		System.out.println(json);
		assertEquals(serialized, json);
	}

	@Test
	public void testDeserialize()
			throws JsonParseException, JsonMappingException, IOException {
		ObjectMapper jackson = new ObjectMapper();
		PartyWithProfile p = jackson.readValue(serialized,
				PartyWithProfile.class);
		System.out.println(p);
		assertEquals(partyprof1, p);
	}

}
