package geniusweb.references;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.references.ProtocolRef;
import tudelft.utilities.junit.GeneralTests;

public class ProtocolRefTest extends GeneralTests<ProtocolRef> {

	private ProtocolRef protocol1, protocol1a, protocol2, protocol3;
	private String asJson = "\"file:StackedAlternatingOffersProtocol\"";

	@Before
	public void before() throws URISyntaxException {
		protocol1 = new ProtocolRef(
				new URI("file:StackedAlternatingOffersProtocol"));
		protocol1a = new ProtocolRef(
				new URI("file:StackedAlternatingOffersProtocol"));
		protocol2 = new ProtocolRef(new URI("ws:localhost/protocol2"));
		protocol3 = new ProtocolRef(new URI("http:localhost/protocol3"));
	}

	@Override
	public List<List<ProtocolRef>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(protocol1, protocol1a),
				Arrays.asList(protocol2), Arrays.asList(protocol3));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("ProtocolRef.*StackedAlternatingOffersProtocol.*",
				"ProtocolRef.*protocol2.*", "ProtocolRef.*protocol3.*");
	}

	@Test
	public void smokeTest() throws URISyntaxException {
		new ProtocolRef(new URI("ws:localhost"));
	}

	@Test(expected = IllegalArgumentException.class)
	public void nullTest() throws URISyntaxException {
		new ProtocolRef((URI) null);
	}

	@Test
	public void testSerialize() throws JsonProcessingException {
		ObjectMapper jackson = new ObjectMapper();

		String json = jackson.writeValueAsString(protocol1);
		System.out.println(json);
		assertEquals(asJson, json);
	}

	@Test
	public void testDeserialize()
			throws JsonParseException, JsonMappingException, IOException {
		ObjectMapper jackson = new ObjectMapper();
		ProtocolRef p = jackson.readValue(asJson, ProtocolRef.class);
		System.out.println(p);
		assertEquals(protocol1, p);
	}

}
